@@ -15,7 +15,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dom Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2010, 2012, 2016-2020 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2010, 2012, 2016-2021 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2011 Vittal Aithal <vittal.aithal@cognidox.com>
 // Copyright (C) 2012, 2013, 2016-2018, 2021 Adrian Johnson <ajohnson@redneon.com>
@@ -43,6 +43,7 @@
 #include <ctime>
 #include <cmath>
 #include <map>
+#include <set>
 #include "parseargs.h"
 #include "printencodings.h"
 #include "goo/GooString.h"
@@ -73,6 +74,7 @@ static int firstPage = 1;
 static int lastPage = 0;
 static bool printBoxes = false;
 static bool printMetadata = false;
+static bool printCustom = false;
 static bool printJS = false;
 static bool isoDates = false;
 static bool rawDates = false;
@@ -90,6 +92,7 @@ static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to
                                    { "-l", argInt, &lastPage, 0, "last page to convert" },
                                    { "-box", argFlag, &printBoxes, 0, "print the page bounding boxes" },
                                    { "-meta", argFlag, &printMetadata, 0, "print the document metadata (XML)" },
+                                   { "-custom", argFlag, &printCustom, 0, "print both custom and standard metadata" },
                                    { "-js", argFlag, &printJS, 0, "print all JavaScript in the PDF" },
                                    { "-struct", argFlag, &printStructure, 0, "print the logical document structure (for tagged files)" },
                                    { "-struct-text", argFlag, &printStructureText, 0, "print text contents along with document structure (for tagged files)" },
@@ -119,6 +122,15 @@ static void printTextString(const GooString *s, const UnicodeMap *uMap)
     gfree(u);
 }
 
+static void printUCS4String(const Unicode *u, int len, const UnicodeMap *uMap)
+{
+    char buf[8];
+    for (int i = 0; i < len; i++) {
+        int n = uMap->mapUnicode(u[i], buf, sizeof(buf));
+        fwrite(buf, 1, n, stdout);
+    }
+}
+
 static void printInfoString(Dict *infoDict, const char *key, const char *text, const UnicodeMap *uMap)
 {
     const GooString *s1;
@@ -634,6 +646,62 @@ static void printPdfSubtype(PDFDoc *doc, const UnicodeMap *uMap)
     }
 }
 
+static void printCustomInfo(PDFDoc *doc, const UnicodeMap *uMap)
+{
+    Object info = doc->getDocInfo();
+    if (info.isDict()) {
+        Dict *dict = info.getDict();
+
+        // Sort keys
+        std::set<std::string> keys;
+        for (int i = 0; i < dict->getLength(); i++) {
+            std::string key(dict->getKey(i));
+            if (key != "Trapped") {
+                keys.insert(key);
+            }
+        }
+
+        for (const std::string &key : keys) {
+            if (key == "CreationDate") {
+                if (isoDates) {
+                    printISODate(info.getDict(), "CreationDate", "CreationDate:    ", uMap);
+                } else if (rawDates) {
+                    printInfoString(info.getDict(), "CreationDate", "CreationDate:    ", uMap);
+                } else {
+                    printInfoDate(info.getDict(), "CreationDate", "CreationDate:    ", uMap);
+                }
+            } else if (key == "ModDate") {
+                if (isoDates) {
+                    printISODate(info.getDict(), "ModDate", "ModDate:         ", uMap);
+                } else if (rawDates) {
+                    printInfoString(info.getDict(), "ModDate", "ModDate:         ", uMap);
+                } else {
+                    printInfoDate(info.getDict(), "ModDate", "ModDate:         ", uMap);
+                }
+            } else {
+                Object obj = dict->lookup(key.c_str());
+                if (obj.isString()) {
+                    // print key
+                    Unicode *u;
+                    int len = utf8ToUCS4(key.c_str(), &u);
+                    printUCS4String(u, len, uMap);
+                    fputs(":", stdout);
+                    while (len < 16) {
+                        fputs(" ", stdout);
+                        len++;
+                    }
+                    gfree(u);
+
+                    // print value
+                    GooString val_str(obj.getString());
+                    printTextString(&val_str, uMap);
+                    fputc('\n', stdout);
+                }
+            }
+        }
+    }
+}
+
 static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, bool multiPage)
 {
     Page *page;
@@ -645,39 +713,63 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
     // print doc info
     Object info = doc->getDocInfo();
     if (info.isDict()) {
-        printInfoString(info.getDict(), "Title", "Title:          ", uMap);
-        printInfoString(info.getDict(), "Subject", "Subject:        ", uMap);
-        printInfoString(info.getDict(), "Keywords", "Keywords:       ", uMap);
-        printInfoString(info.getDict(), "Author", "Author:         ", uMap);
-        printInfoString(info.getDict(), "Creator", "Creator:        ", uMap);
-        printInfoString(info.getDict(), "Producer", "Producer:       ", uMap);
+        printInfoString(info.getDict(), "Title", "Title:           ", uMap);
+        printInfoString(info.getDict(), "Subject", "Subject:         ", uMap);
+        printInfoString(info.getDict(), "Keywords", "Keywords:        ", uMap);
+        printInfoString(info.getDict(), "Author", "Author:          ", uMap);
+        printInfoString(info.getDict(), "Creator", "Creator:         ", uMap);
+        printInfoString(info.getDict(), "Producer", "Producer:        ", uMap);
         if (isoDates) {
-            printISODate(info.getDict(), "CreationDate", "CreationDate:   ", uMap);
-            printISODate(info.getDict(), "ModDate", "ModDate:        ", uMap);
+            printISODate(info.getDict(), "CreationDate", "CreationDate:    ", uMap);
+            printISODate(info.getDict(), "ModDate", "ModDate:         ", uMap);
         } else if (rawDates) {
-            printInfoString(info.getDict(), "CreationDate", "CreationDate:   ", uMap);
-            printInfoString(info.getDict(), "ModDate", "ModDate:        ", uMap);
+            printInfoString(info.getDict(), "CreationDate", "CreationDate:    ", uMap);
+            printInfoString(info.getDict(), "ModDate", "ModDate:         ", uMap);
         } else {
-            printInfoDate(info.getDict(), "CreationDate", "CreationDate:   ", uMap);
-            printInfoDate(info.getDict(), "ModDate", "ModDate:        ", uMap);
+            printInfoDate(info.getDict(), "CreationDate", "CreationDate:    ", uMap);
+            printInfoDate(info.getDict(), "ModDate", "ModDate:         ", uMap);
         }
     }
 
+    bool hasMetadata = false;
+    std::unique_ptr<GooString> metadata = doc->readMetadata();
+    if (metadata) {
+        hasMetadata = true;
+    }
+
+    const std::set<std::string> docInfoStandardKeys { "Title", "Author", "Subject", "Keywords", "Creator", "Producer", "CreationDate", "ModDate", "Trapped" };
+
+    bool hasCustom = false;
+    if (info.isDict()) {
+        Dict *dict = info.getDict();
+        for (i = 0; i < dict->getLength(); i++) {
+            std::string key(dict->getKey(i));
+            if (docInfoStandardKeys.find(key) == docInfoStandardKeys.end()) {
+                hasCustom = true;
+                break;
+            }
+        }
+    }
+
+    // print metadata info
+    printf("Custom Metadata: %s\n", hasCustom ? "yes" : "no");
+    printf("Metadata Stream: %s\n", hasMetadata ? "yes" : "no");
+
     // print tagging info
-    printf("Tagged:         %s\n", (doc->getCatalog()->getMarkInfo() & Catalog::markInfoMarked) ? "yes" : "no");
-    printf("UserProperties: %s\n", (doc->getCatalog()->getMarkInfo() & Catalog::markInfoUserProperties) ? "yes" : "no");
-    printf("Suspects:       %s\n", (doc->getCatalog()->getMarkInfo() & Catalog::markInfoSuspects) ? "yes" : "no");
+    printf("Tagged:          %s\n", (doc->getCatalog()->getMarkInfo() & Catalog::markInfoMarked) ? "yes" : "no");
+    printf("UserProperties:  %s\n", (doc->getCatalog()->getMarkInfo() & Catalog::markInfoUserProperties) ? "yes" : "no");
+    printf("Suspects:        %s\n", (doc->getCatalog()->getMarkInfo() & Catalog::markInfoSuspects) ? "yes" : "no");
 
     // print form info
     switch (doc->getCatalog()->getFormType()) {
     case Catalog::NoForm:
-        printf("Form:           none\n");
+        printf("Form:            none\n");
         break;
     case Catalog::AcroForm:
-        printf("Form:           AcroForm\n");
+        printf("Form:            AcroForm\n");
         break;
     case Catalog::XfaForm:
-        printf("Form:           XFA\n");
+        printf("Form:            XFA\n");
         break;
     }
 
@@ -685,14 +777,14 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
     {
         JSInfo jsInfo(doc, firstPage - 1);
         jsInfo.scanJS(lastPage - firstPage + 1);
-        printf("JavaScript:     %s\n", jsInfo.containsJS() ? "yes" : "no");
+        printf("JavaScript:      %s\n", jsInfo.containsJS() ? "yes" : "no");
     }
 
     // print page count
-    printf("Pages:          %d\n", doc->getNumPages());
+    printf("Pages:           %d\n", doc->getNumPages());
 
     // print encryption info
-    printf("Encrypted:      ");
+    printf("Encrypted:       ");
     if (doc->isEncrypted()) {
         unsigned char *fileKey;
         CryptAlgorithm encAlgorithm;
@@ -725,9 +817,9 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
         w = doc->getPageCropWidth(pg);
         h = doc->getPageCropHeight(pg);
         if (multiPage) {
-            printf("Page %4d size: %g x %g pts", pg, w, h);
+            printf("Page %4d size:  %g x %g pts", pg, w, h);
         } else {
-            printf("Page size:      %g x %g pts", w, h);
+            printf("Page size:       %g x %g pts", w, h);
         }
         if ((fabs(w - 612) < 1 && fabs(h - 792) < 1) || (fabs(w - 792) < 1 && fabs(h - 612) < 1)) {
             printf(" (letter)");
@@ -748,9 +840,9 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
         printf("\n");
         r = doc->getPageRotate(pg);
         if (multiPage) {
-            printf("Page %4d rot:  %d\n", pg, r);
+            printf("Page %4d rot:   %d\n", pg, r);
         } else {
-            printf("Page rot:       %d\n", r);
+            printf("Page rot:        %d\n", r);
         }
     }
 
@@ -763,15 +855,15 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
                     error(errSyntaxError, -1, "Failed to print boxes for page {0:d}", pg);
                     continue;
                 }
-                sprintf(buf, "Page %4d MediaBox: ", pg);
+                sprintf(buf, "Page %4d MediaBox:  ", pg);
                 printBox(buf, page->getMediaBox());
-                sprintf(buf, "Page %4d CropBox:  ", pg);
+                sprintf(buf, "Page %4d CropBox:   ", pg);
                 printBox(buf, page->getCropBox());
-                sprintf(buf, "Page %4d BleedBox: ", pg);
+                sprintf(buf, "Page %4d BleedBox:  ", pg);
                 printBox(buf, page->getBleedBox());
-                sprintf(buf, "Page %4d TrimBox:  ", pg);
+                sprintf(buf, "Page %4d TrimBox:   ", pg);
                 printBox(buf, page->getTrimBox());
-                sprintf(buf, "Page %4d ArtBox:   ", pg);
+                sprintf(buf, "Page %4d ArtBox:    ", pg);
                 printBox(buf, page->getArtBox());
             }
         } else {
@@ -779,23 +871,23 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
             if (!page) {
                 error(errSyntaxError, -1, "Failed to print boxes for page {0:d}", firstPage);
             } else {
-                printBox("MediaBox:       ", page->getMediaBox());
-                printBox("CropBox:        ", page->getCropBox());
-                printBox("BleedBox:       ", page->getBleedBox());
-                printBox("TrimBox:        ", page->getTrimBox());
-                printBox("ArtBox:         ", page->getArtBox());
+                printBox("MediaBox:        ", page->getMediaBox());
+                printBox("CropBox:         ", page->getCropBox());
+                printBox("BleedBox:        ", page->getBleedBox());
+                printBox("TrimBox:         ", page->getTrimBox());
+                printBox("ArtBox:          ", page->getArtBox());
             }
         }
     }
 
     // print file size
-    printf("File size:      %lld bytes\n", filesize);
+    printf("File size:       %lld bytes\n", filesize);
 
     // print linearization info
-    printf("Optimized:      %s\n", doc->isLinearized() ? "yes" : "no");
+    printf("Optimized:       %s\n", doc->isLinearized() ? "yes" : "no");
 
     // print PDF version
-    printf("PDF version:    %d.%d\n", doc->getPDFMajorVersion(), doc->getPDFMinorVersion());
+    printf("PDF version:     %d.%d\n", doc->getPDFMajorVersion(), doc->getPDFMinorVersion());
 
     printPdfSubtype(doc, uMap);
 }
@@ -902,12 +994,13 @@ int main(int argc, char *argv[])
 
     if (printMetadata) {
         // print the metadata
-        const GooString *metadata = doc->readMetadata();
+        const std::unique_ptr<GooString> metadata = doc->readMetadata();
         if (metadata) {
             fputs(metadata->c_str(), stdout);
             fputc('\n', stdout);
-            delete metadata;
         }
+    } else if (printCustom) {
+        printCustomInfo(doc.get(), uMap);
     } else if (printJS) {
         // print javascript
         JSInfo jsInfo(doc.get(), firstPage - 1);
