@@ -37,6 +37,7 @@
 // Copyright (C) 2020, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
 // Copyright (C) 2020 Salvo Miosi <salvo.ilmiosi@gmail.com>
+// Copyright (C) 2021 Peter Williams <peter@newton.cx>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -50,6 +51,9 @@
 #include <cmath>
 #include <cstring>
 #include <fcntl.h>
+#if defined(_WIN32) || defined(__CYGWIN__)
+#    include <io.h> // for _setmode
+#endif
 #include "parseargs.h"
 #include "goo/gmem.h"
 #include "goo/GooString.h"
@@ -388,8 +392,8 @@ static void writePageImage(GooString *filename)
         return;
 
     if (filename->cmp("fd://0") == 0) {
-#ifdef _WIN32
-        setmode(fileno(stdout), O_BINARY);
+#if defined(_WIN32) || defined(__CYGWIN__)
+        _setmode(fileno(stdout), O_BINARY);
 #endif
         file = stdout;
     } else
@@ -558,9 +562,12 @@ static void beginDocument(GooString *inputFileName, GooString *outputFileName, d
         if (printToWin32) {
             output_file = nullptr;
         } else {
-            if (outputFileName->cmp("fd://0") == 0)
+            if (outputFileName->cmp("fd://0") == 0) {
+#if defined(_WIN32) || defined(__CYGWIN__)
+                _setmode(fileno(stdout), O_BINARY);
+#endif
                 output_file = stdout;
-            else {
+            } else {
                 output_file = fopen(outputFileName->c_str(), "wb");
                 if (!output_file) {
                     fprintf(stderr, "Error opening output file %s\n", outputFileName->c_str());
